/**********************************************************************
** This program generates a distribution of total synaptic input
** assuming facilitating synapses where the release probability changes
** with history.
**           Copyright (C) 2007 Upinder S. Bhalla. and NCBS
** It is made available under the terms of the
** GNU Lesser General Public License version 2.1,
** which permits free distribution without warranty of any kind.
** See the file COPYING.LIB for the full notice.
**********************************************************************/
#include <iostream>
#include <fstream>
#include <vector>
#include <cassert>
#include <math.h>
#include "randnum.h"

using namespace std;

static const unsigned int NUM_CYCLES = 30000;

/*
 * This program generates a distribution of total synaptic input
 * assuming facilitating synapses where the release probability changes
 * with history. For now we only consider a specific spacing of release
 * and we don't yet worry about release amplitude issues.
 * This program generates output either for a single pulse (p1 = 0)
 * or for 2 inputs (p1s are > 0), applied to all axons.
 */

int main( int argc, char** argv )
{
	if ( argc < 5 ) {
		cout << "usage: " << argv[0] << " numAxons p0 p1_no_prev p1_yes_prev \n";
		return 0;
	}
	unsigned int numAxons = atoi( argv[1] );
	// Prob of first release
	double p0 = atof( argv[2] ); 	

	// Prob of release if nothing released yet
	double p1_no_prev = atof( argv[3] );	

	// Prob of release if previous release has happened.
	double p1_yes_prev = atof( argv[4] );	

	double sum = 0.0;
	double sumsq = 0.0;

	mtseed( 0 ); // Set up an initial, pretty random seed.
	for ( unsigned int j = 0; j < NUM_CYCLES; j++ ) {
		double total = 0.0;
		for (unsigned int i = 0; i < numAxons; i++ ) {
			bool yes_prev = 0;
			if ( mtrand() < p0 ) {
				total += 1.0;
				yes_prev = 1;
			}
			if ( yes_prev ) {
				if ( p1_yes_prev > 0 ) {
					if ( mtrand() < p1_yes_prev ) {
						total += 1.0;
					}
				}
			} else {
				if ( p1_no_prev > 0 ) {
					if ( mtrand() < p1_no_prev )
						total += 1.0;
				}
			}
		}
		// cout << total << endl;
		sum += total;
		sumsq += total * total;
	}
	cout << numAxons << "	" << p0 << "	" << p1_no_prev << "	" <<
		p1_yes_prev << "	" << sum / NUM_CYCLES << "	"	<<
		sqrt ( ( sumsq - sum * sum / NUM_CYCLES ) / NUM_CYCLES ) << endl;
}
